"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  render: true,
  cleanup: true,
  fireEvent: true,
  wait: true,
  flush: true
};
exports.render = render;
exports.cleanup = cleanup;
exports.fireEvent = fireEvent;
exports.wait = wait;
exports.flush = flush;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _angular = _interopRequireDefault(require("angular"));

require("angular-mocks");

var _dom = require("@testing-library/dom");

Object.keys(_dom).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _dom[key];
    }
  });
});
const mountedContainers = new Set();
const mountedScopes = new Set();

function render(ui, {
  container,
  baseElement = container,
  queries,
  scope
} = {}) {
  if (!baseElement) {
    // default to document.body instead of documentElement to avoid output of potentially-large
    // head elements (such as JSS style blocks) in debug output
    baseElement = document.body;
  }

  if (!container) {
    container = baseElement.appendChild(document.createElement('div'));
  } // we'll add it to the mounted containers regardless of whether it's actually
  // added to document.body so the cleanup method works regardless of whether
  // they're passing us a custom container or not.


  mountedContainers.add(container);
  const $rootScope = getAngularService('$rootScope');
  const $compile = getAngularService('$compile');
  const $scope = $rootScope.$new();
  Object.assign($scope, scope);
  mountedScopes.add($scope);
  const element = $compile(ui)($scope)[0];
  container.appendChild(element);
  $scope.$digest();
  return (0, _extends2.default)({
    container,
    baseElement,
    debug: (el = baseElement) => Array.isArray(el) ? // eslint-disable-next-line no-console
    el.forEach(e => console.log((0, _dom.prettyDOM)(e))) : // eslint-disable-next-line no-console,
    console.log((0, _dom.prettyDOM)(el)),
    asFragment: () => {
      /* istanbul ignore if (jsdom limitation) */
      if (typeof document.createRange === 'function') {
        return document.createRange().createContextualFragment(container.innerHTML);
      }

      const template = document.createElement('template');
      template.innerHTML = container.innerHTML;
      return template.content;
    },
    $scope,
    unmount: () => {
      $scope.$destroy();
    }
  }, (0, _dom.getQueriesForElement)(baseElement, queries));
}

function cleanup() {
  mountedScopes.forEach(cleanupScope);
  mountedContainers.forEach(cleanupAtContainer);
} // maybe one day we'll expose this (perhaps even as a utility returned by render).
// but let's wait until someone asks for it.


function cleanupAtContainer(container) {
  if (container.parentNode === document.body) {
    document.body.removeChild(container);
  }

  mountedContainers.delete(container);
}

function cleanupScope(scope) {
  scope.$destroy();
  mountedScopes.delete(scope);
}

function getAngularService(name) {
  let service;

  _angular.default.mock.inject([name, injected => {
    service = injected;
  }]);

  return service;
}

function fireEvent(...args) {
  const $rootScope = getAngularService('$rootScope');
  const result = (0, _dom.fireEvent)(...args);
  $rootScope.$digest();
  return result;
}

Object.keys(_dom.fireEvent).forEach(key => {
  fireEvent[key] = (...args) => {
    const $rootScope = getAngularService('$rootScope');

    const result = _dom.fireEvent[key](...args);

    $rootScope.$digest();
    return result;
  };
}); // AngularJS maps `mouseEnter` to `mouseOver` and `mouseLeave` to `mouseOut`

fireEvent.mouseEnter = fireEvent.mouseOver;
fireEvent.mouseLeave = fireEvent.mouseOut;

function flush(millis = 50) {
  const $browser = getAngularService('$browser');
  const $rootScope = getAngularService('$rootScope');
  const $interval = getAngularService('$interval');
  $interval.flush(millis);
  $browser.defer.flush(millis);
  $rootScope.$digest();
}

function wait(callback, options = {}) {
  return (0, _dom.wait)(() => {
    flush(options.interval);

    if (callback) {
      callback();
    }
  }, options);
}